Package: RODBC
Version: 1.3-26.1
Revision: $Rev: 3689 $
Date: 2025-12-15
Authors@R: c(person("Brian", "Ripley", role = c("aut", "cre"),
                    email = "Brian.Ripley@R-project.org"),
             person("Michael", "Lapsley", role = "aut", 
	            comment = "1999 to Oct 2002"))
Title: ODBC Database Access
Description: An ODBC database interface.
SystemRequirements: An ODBC3 driver manager and drivers.
Depends: R (>= 4.0.0)
Imports: stats
LazyLoad: yes
Biarch: yes
License: GPL-2 | GPL-3
NeedsCompilation: yes
Packaged: 2025-12-15 11:32:40 UTC; ripley
Author: Brian Ripley [aut, cre],
  Michael Lapsley [aut] (1999 to Oct 2002)
Maintainer: Brian Ripley <Brian.Ripley@R-project.org>
Repository: CRAN
Date/Publication: 2025-12-15 12:32:58
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 16:49:45 UTC; unix
Archs: RODBC.so.dSYM
