useDynLib(RODBC, .registration = TRUE, .fixes = "C_")

importFrom(stats, runif)
importFrom(utils, type.convert)

export(getSqlTypeInfo, odbcClearError, odbcClose, odbcCloseAll,
       odbcConnect, odbcDataSources, odbcDriverConnect, odbcEndTran,
       odbcFetchRows, odbcGetErrMsg, odbcGetInfo, odbcQuery,
       odbcReConnect, odbcSetAutoCommit, odbcTables, odbcUpdate,
       setSqlTypeInfo, sqlClear, sqlColumns, sqlCopy, sqlCopyTable,
       sqlDrop, sqlFetch, sqlFetchMore, sqlGetResults, sqlPrimaryKeys,
       sqlQuery, sqlSave, sqlTables, sqlTypeInfo, sqlUpdate)

S3method(close, RODBC)
S3method(print, RODBC)
S3method(format, "ODBC_binary")
S3method(print, "ODBC_binary")

if(tools:::.OStype() == "windows") {
export(odbcConnectAccess, odbcConnectAccess2007, odbcConnectDbase,
       odbcConnectExcel, odbcConnectExcel2007)
}
